//Preamble

//Endogenous variables 
//y: Output 
//c: Consumption 
//n: Employment 
//w: Real wage 
//mc: Real marginal costs 
//pi: Producer price inflation 
//pi_c: Consumer price inflation 
//i: Union-wide nominal interest rate 
//r: Real interest rate 
//s: Terms of trade 
//b: Real asset holdings
//mu: Utility shadow price
//mrs: Marginal rate of substitution
//a: Labor productivity 
//g: Government expenditures 
//v: Disturbances to monetary policy
//e_u: Preference shock
//e_n: Labor supply shock

//Exogenous variables 
//epsilon_a: Technology shock
//epsilon_g: Fiscal policy shock 
//epsilon_v: Monetary policy shock
//epsilon_u: Preference shock
//epsilon_n: Labor supply shock

//Parameters 

//alpha: Degree of openness 
//beta: Utility discount factor 
//rho: -log(beta)
//theta: Degree of price-stickiness/Calvo parameter 
//sigma: Degree of risk-aversion with respect to consumption 
//eta: Elasticity of substitution between foreign and home goods
//phi: Degree of risk-aversion with respect to leisure 
//rho_v: Persistence of the monetary policy shock 
//rho_a: Persistence of the technology shock 
//rho_g: Persistence of the fiscal policy shock 
//rho_e_u: Persistence of the preference shock 
//rho_e_n: Persistence of the labor supply shock 
//rho_g: Persistence of the fiscal policy shock 
//phi_y: Weight the central bank puts on stabilizing output 
//phi_y: Weight the central bank puts on stabilizing inflation
//phi_r: Interest rate smoothing in the Taylor rule
//n_tilde: GDP weight in the aggregate variables
//gamma: Consumption habits
//chi_pi: Inflation indexation
//chi_w: Real wage rigidity


//Endogenous variables

var y_H_hat, y_F_hat, pi_H, pi_F_star, pi_c_H, pi_c_F_star, r_H, i_H_hat, i_F_hat, r_F, a_H, a_F, e_u_H, e_u_F,  e_n_H, e_n_F, g_H_hat, g_F_hat, v_H, v_F, c_H_hat, c_H_H_hat, c_H_F_hat, w_H_hat, n_H_hat, 
c_F_hat, c_F_F_hat, c_F_H_hat, w_F_hat, n_F_hat, mc_H_hat, mc_F_hat, s, mu_H_hat, mu_F_hat, mrs_H_hat, mrs_F_hat, i_H, i_F, cpi_H, cpi_F_star, p_H, p_F_star, e;


//Exogenous variables

varexo epsilon_a_H epsilon_a_F epsilon_v_H epsilon_v_F epsilon_e_u_H epsilon_e_u_F epsilon_g_H epsilon_g_F epsilon_e_n_H epsilon_e_n_F;


//Parameters

parameters alpha_H, alpha_F, beta, theta_H, theta_F, sigma, phi_H, phi_F, eta_H, eta_F, rho_v_H, rho_v_F, rho_a_H, rho_a_F, rho_g_H, rho_g_F, rho_e_u_H, rho_e_u_F, rho_e_n_H, rho_e_n_F, rho_e_c_H, rho_e_c_F, phi_y_H, phi_pi_H, phi_y_F, phi_pi_F, n_tilde_H, n_tilde_F, phi_r_H,phi_r_F,  
rho, chi_pi_H, chi_pi_F, chi_w_H, chi_w_F, 
 y_ss,omega;


//Calibration

alpha_H = 0.15;
alpha_F = 0.1;
beta = 0.998;
theta_H = 0.7;
theta_F = 0.7;
sigma = 1;
phi_H = 2;
phi_F = 2;
eta_H = 1.1;
eta_F = 1.1;
gamma_H = 0.7;
gamma_F = 0.7;
chi_w_H = 0.9;
chi_w_F = 0.9;
chi_pi_H = 0.7;
chi_pi_F = 0.7;

rho_v_H = 0;
rho_v_F = 0;
rho_a_H = 0.7;
rho_a_F = 0.7;
rho_g_H = 0.9;
rho_g_F = 0.9;
rho_e_u_H = 0.9;
rho_e_u_F = 0.9;
rho_e_n_H = 0.9;
rho_e_n_F = 0.9;
rho_e_c_H = 0.9;
rho_e_c_F = 0.9;

phi_y_H = 0.2;
phi_y_F = 0.2;
phi_pi_H = 1.6;
phi_pi_F = 1.6;


n_tilde_H = 0.3;
n_tilde_F = 0.7;
phi_r_H = 0.8;
phi_r_F = 0.8;

rho = -1*log(beta);

//y_ss = 1;
//c_ss = omega_c*y_ss;
//g_ss = omega_g*y_ss;
omega = (sigma + (1-alpha_H - alpha_F)*(sigma-1))/sigma;


model (linear);


//////////////
//Country H://
//////////////


//Households

mu_H_hat = -sigma*c_H_hat + e_u_H;

e_u_H + e_n_H + phi_H*n_H_hat = mu_H_hat + mrs_H_hat;

mu_H_hat = i_H - rho - pi_c_H(+1) + mu_H_hat(+1);
w_H_hat = (1-chi_w_H)*mrs_H_hat + chi_w_H*w_H_hat(-1);


//Firms and Price Setting

y_H_hat = a_H + n_H_hat;
pi_H - chi_pi_H*pi_H(-1) = beta*(pi_H(+1) - chi_pi_H*pi_H) + (((1-theta_H*beta)*(1-theta_H))/theta_H)*(mc_H_hat);
mc_H_hat = w_H_hat + alpha_H*s - a_H;


//Aggregate consistency

y_H_hat= g_H_hat + c_H_hat + alpha_H*omega*s;

c_H_H_hat = eta_H*alpha_H*s + c_H_hat;
c_H_F_hat = eta_F*(1-alpha_F)*s + c_F_hat;


//Supplemental equations

pi_c_H = pi_H + alpha_H*(s-s(-1));
r_H = i_H - rho - pi_c_H(+1);


//////////////
//Country F://
//////////////


//Households

mu_F_hat = -sigma*c_F_hat + e_u_F;

e_u_F + e_n_F + phi_F*n_F_hat = mu_F_hat + mrs_F_hat;

mu_F_hat = i_F - rho - pi_c_F_star(+1) + mu_F_hat(+1);
w_F_hat = (1-chi_w_F)*mrs_F_hat + chi_w_F*w_F_hat(-1);


//Firms and Price setting

y_F_hat = a_F + n_F_hat;
pi_F_star - chi_pi_F*pi_F_star(-1) = beta*(pi_F_star(+1) - chi_pi_F*pi_F_star) + (((1-theta_F*beta)*(1-theta_F))/theta_F)*(mc_F_hat);
mc_F_hat = w_F_hat - alpha_F*s - a_F;


//Aggregate consistency

y_F_hat= g_F_hat + c_F_hat - alpha_F*omega*s;

c_F_F_hat = (-1)*eta_F*alpha_F*s + c_F_hat;
c_F_H_hat = (-1)*eta_H*(1-alpha_H)*s + c_H_hat;


//Supplemental equations

pi_c_F_star = pi_F_star - alpha_F*(s-s(-1));
r_F = i_F - rho - pi_c_F_star(+1);


///////////////////////////////////////////
//Connection between the two economies:////
///////////////////////////////////////////


//Central bank 1

i_H_hat = phi_r_H*i_H_hat(-1) + (1-phi_r_H)*(phi_y_H*y_H_hat + phi_pi_H*pi_c_H) + v_H;
i_H_hat = i_H - rho;

//Central bank 2

i_F_hat = phi_r_F*i_F_hat(-1) + (1-phi_r_F)*(phi_y_F*y_F_hat + phi_pi_F*pi_c_F_star) + v_F;
i_F_hat = i_F - rho;


//Net foreign asset position

y_H_hat = y_F_hat + g_H_hat - g_F_hat + ((sigma*omega*(alpha_F + alpha_H) + (1-alpha_H- alpha_F))/sigma)*s;



//Prices

cpi_F_star= (1-alpha_F)*s + p_H - e;

pi_c_H= cpi_H - cpi_H(-1);
pi_c_F_star= cpi_F_star - cpi_F_star(-1);
pi_F_star= p_F_star- p_F_star(-1);
pi_H= p_H- p_H(-1);


/////////////
//Shocks;////
/////////////

a_H = rho_a_H*a_H(-1) + epsilon_a_H;
a_F = rho_a_F*a_F(-1) + epsilon_a_F;
g_H_hat = rho_g_H*g_H_hat(-1) + epsilon_g_H;
g_F_hat = rho_g_F*g_F_hat(-1) + epsilon_g_F;
v_H = rho_v_H*v_H(-1) + epsilon_v_H;
v_F = rho_v_F*v_F(-1) + epsilon_v_F;
e_u_H = rho_e_u_H*e_u_H(-1) + epsilon_e_u_H;
e_u_F = rho_e_u_F*e_u_F(-1) + epsilon_e_u_F;
e_n_H = rho_e_n_H*e_n_H(-1) + epsilon_e_n_H;
e_n_F = rho_e_n_F*e_n_F(-1) + epsilon_e_n_F;
end;

initval;
 
y_H_hat = 0;
y_F_hat = 0;
pi_H = 0;
pi_F_star = 0;
pi_c_H = 0;
pi_c_F_star = 0;
c_H_hat = 0;
c_H_H_hat = 0;
c_H_F_hat = 0;
c_F_hat = 0;
c_F_F_hat = 0;
c_F_H_hat = 0;
mc_H_hat = 0;
mc_F_hat = 0;
i_H = rho;
i_H_hat = 0;
i_F = rho;
i_F_hat = 0;
s = 0;
g_H_hat = 0;
g_F_hat = 0;
a_H = 0;
a_F = 0;
v_H = 0;
v_F = 0;
e_u_H = 0;
e_u_F = 0;
e_n_H = 0;
e_n_F = 0;
n_H_hat = 0;
n_F_hat = 0;
w_H_hat = 0;
w_F_hat = 0;
r_H = 0;
r_F = 0;
mrs_H_hat = 0;
mrs_F_hat = 0;
mu_H_hat = 0;
mu_F_hat = 0;

end;

steady;

check;

shocks;

var epsilon_a_H = 0.01^2;
var epsilon_a_F = 0.01^2;
var epsilon_v_H = 0.01^2;
var epsilon_v_F = 0.01^2;
var epsilon_e_u_H = 0.1^2;
var epsilon_e_u_F = 0.1^2;
var epsilon_g_H = 0.01^2;
var epsilon_g_F = 0.01^2;
var epsilon_e_n_H = 0.1^2;
var epsilon_e_n_F = 0.1^2;


end;

estimated_params;

sigma, normal_pdf, 1, 0.375;
phi_H, normal_pdf, 2, 0.2;
phi_F, normal_pdf, 2, 0.2;
theta_H, beta_pdf, 0.7, 0.05;
theta_F, beta_pdf, 0.7, 0.05;
//gamma_H, beta_pdf, 0.7, 0.1;
//gamma_F, beta_pdf, 0.7, 0.1;
chi_w_H, beta_pdf, 0.9, 0.025;
chi_w_F, beta_pdf, 0.9, 0.025;
chi_pi_H, beta_pdf, 0.7, 0.15;
chi_pi_F, beta_pdf, 0.7, 0.15;
phi_y_H, normal_pdf, 0.2, 0.05;
phi_y_F, normal_pdf, 0.2, 0.05;
phi_pi_H, normal_pdf, 1.6, 0.1;
phi_pi_F, normal_pdf, 1.6, 0.1;
phi_r_H, beta_pdf, 0.8, 0.05;
phi_r_F, beta_pdf, 0.8, 0.05;
rho_a_H, beta_pdf, 0.85, 0.1;
rho_a_F, beta_pdf, 0.85, 0.1;
rho_e_u_H, beta_pdf, 0.85, 0.1;
rho_e_u_F, beta_pdf, 0.85, 0.1;
rho_e_n_H, beta_pdf, 0.85, 0.1;
rho_e_n_F, beta_pdf, 0.85, 0.1;
rho_g_H, beta_pdf, 0.85, 0.1;
rho_g_F, beta_pdf, 0.85, 0.1;
stderr epsilon_a_H, inv_gamma_pdf, 0.02, inf;
stderr epsilon_a_F, inv_gamma_pdf, 0.02, inf;
stderr epsilon_e_u_H, inv_gamma_pdf, 0.02, inf;
stderr epsilon_e_u_F, inv_gamma_pdf, 0.02, inf;
stderr epsilon_e_n_H, inv_gamma_pdf, 0.05, inf;
stderr epsilon_e_n_F, inv_gamma_pdf, 0.05, inf;
stderr epsilon_g_H, inv_gamma_pdf, 0.02, inf;
stderr epsilon_g_F, inv_gamma_pdf, 0.02, inf;
stderr epsilon_v_H, inv_gamma_pdf, 0.001, inf;
stderr epsilon_v_F, inv_gamma_pdf, 0.001, inf;

end;

estimated_params_init(use_calibration);

//sigma,  1.4167;
//phi_H,  1.8477;
//phi_F,  1.8118;
//theta_H,  0.5485;
//theta_F, 0.6793;
//gamma_H, 0.6593;
//gamma_F,  0.7435;
//chi_w_H,  0.8998;
//chi_w_F,  0.8902;
//chi_pi_H, 0.2698;
//chi_pi_F,  0.257;
//psi, 0.0019;
//phi_y_H,  0.191;
//phi_y_F,  0.191;
//phi_pi_H,  1.5477;
//phi_pi_F,  1.5477;
//phi_r_H,  0.8305;
//phi_r_F,  0.8305;
//rho_a_H, 0.5538;
//rho_a_F,  0.7462;
//rho_e_u_H,  0.7596;
//rho_e_u_F,  0.8074;
//rho_e_n_H,  0.3649;
//rho_e_n_F,  0.3329;
//rho_g_H,  0.8203;
//rho_g_F,  0.84;
//stderr epsilon_a_H,  0.016;
//stderr epsilon_a_F,  0.0102;
//stderr epsilon_e_u_H,  0.0107;
//stderr epsilon_e_u_F,  0.0089;
//stderr epsilon_e_n_H,  0.0554;
//stderr epsilon_e_n_F,  0.0407;
//stderr epsilon_g_H, 0.027;
//stderr epsilon_g_F, 0.014;
//stderr epsilon_v_H, 0.0008;
//stderr epsilon_v_F, 0.0008;

end;

//identification;

//stoch_simul(irf=20) y_H_hat pi_c_H c_H_hat y_F_hat pi_c_F_star c_F_hat;

varobs w_F_hat y_F_hat c_F_hat pi_F_star i_F w_H_hat y_H_hat c_H_hat pi_H i_H;

estimation(datafile=US_D, bayesian_irf, moments_varendo, conditional_variance_decomposition=[1:10], xls_range= a1:j185, mh_nblocks = 2, mh_jscale = 0.36, mh_replic =1000000, mode_check, mode_compute = 9, forecast=10) w_F_hat y_F_hat c_F_hat pi_F_star i_F w_H_hat y_H_hat c_H_hat pi_H i_H;